@@ -130,7 +130,7 @@ class _PropertyMixin(object):
         :param client: the client to use.  If not passed, falls back to the
                        ``client`` stored on the current object.
         :type timeout: float or tuple
-        :param timeout: (optional) The amount of time, in seconds, to wait
+        :param timeout: (Optional) The amount of time, in seconds, to wait
             for the server response.
 
             Can also be passed as a tuple (connect_timeout, read_timeout).
@@ -191,7 +191,7 @@ class _PropertyMixin(object):
         :param client: the client to use.  If not passed, falls back to the
                        ``client`` stored on the current object.
         :type timeout: float or tuple
-        :param timeout: (optional) The amount of time, in seconds, to wait
+        :param timeout: (Optional) The amount of time, in seconds, to wait
             for the server response.
 
             Can also be passed as a tuple (connect_timeout, read_timeout).
@@ -227,7 +227,7 @@ class _PropertyMixin(object):
         :param client: the client to use.  If not passed, falls back to the
                        ``client`` stored on the current object.
         :type timeout: float or tuple
-        :param timeout: (optional) The amount of time, in seconds, to wait
+        :param timeout: (Optional) The amount of time, in seconds, to wait
             for the server response.
 
             Can also be passed as a tuple (connect_timeout, read_timeout).
