@@ -146,26 +146,26 @@ class LifecycleRuleConditions(dict):
     See: https://cloud.google.com/storage/docs/lifecycle
 
     :type age: int
-    :param age: (optional) apply rule action to items whos age, in days,
+    :param age: (Optional) Apply rule action to items whos age, in days,
                 exceeds this value.
 
     :type created_before: datetime.date
-    :param created_before: (optional) apply rule action to items created
+    :param created_before: (Optional) Apply rule action to items created
                            before this date.
 
     :type is_live: bool
-    :param is_live: (optional) if true, apply rule action to non-versioned
+    :param is_live: (Optional) If true, apply rule action to non-versioned
                     items, or to items with no newer versions. If false, apply
                     rule action to versioned items with at least one newer
                     version.
 
     :type matches_storage_class: list(str), one or more of
                                  :attr:`Bucket.STORAGE_CLASSES`.
-    :param matches_storage_class: (optional) apply rule action to items which
+    :param matches_storage_class: (Optional) Apply rule action to items which
                                   whose storage class matches this value.
 
     :type number_of_newer_versions: int
-    :param number_of_newer_versions: (optional) apply rule action to versioned
+    :param number_of_newer_versions: (Optional) Apply rule action to versioned
                                      items having N newer versions.
 
     :raises ValueError: if no arguments are passed.
@@ -316,11 +316,11 @@ class IAMConfiguration(dict):
 
     :type uniform_bucket_level_access_enabled: bool
     :params bucket_policy_only_enabled:
-        (optional) whether the IAM-only policy is enabled for the bucket.
+        (Optional) Whether the IAM-only policy is enabled for the bucket.
 
     :type uniform_bucket_level_locked_time: :class:`datetime.datetime`
     :params uniform_bucket_level_locked_time:
-        (optional) When the bucket's IAM-only policy was enabled.
+        (Optional) When the bucket's IAM-only policy was enabled.
         This value should normally only be set by the back-end API.
 
     :type bucket_policy_only_enabled: bool
@@ -549,7 +549,7 @@ class Bucket(_PropertyMixin):
 
         :type client: :class:`~google.cloud.storage.client.Client` or
                       ``NoneType``
-        :param client: Optional. The client to use.
+        :param client: (Optional) The client to use.
 
         :rtype: :class:`google.cloud.storage.bucket.Bucket`
         :returns: The bucket object created.
@@ -593,14 +593,14 @@ class Bucket(_PropertyMixin):
 
         :type encryption_key: bytes
         :param encryption_key:
-            Optional 32 byte encryption key for customer-supplied encryption.
+            (Optional) 32 byte encryption key for customer-supplied encryption.
 
         :type kms_key_name: str
         :param kms_key_name:
-            Optional resource name of KMS key used to encrypt blob's content.
+            (Optional) Resource name of KMS key used to encrypt blob's content.
 
         :type generation: long
-        :param generation: Optional. If present, selects a specific revision of
+        :param generation: (Optional) If present, selects a specific revision of
                            this object.
 
         :rtype: :class:`google.cloud.storage.blob.Blob`
@@ -617,12 +617,13 @@ class Bucket(_PropertyMixin):
 
     def notification(
         self,
-        topic_name,
+        topic_name=None,
         topic_project=None,
         custom_attributes=None,
         event_types=None,
         blob_name_prefix=None,
         payload_format=NONE_PAYLOAD_FORMAT,
+        notification_id=None,
     ):
         """Factory:  create a notification resource for the bucket.
 
@@ -632,12 +633,13 @@ class Bucket(_PropertyMixin):
         """
         return BucketNotification(
             self,
-            topic_name,
+            topic_name=topic_name,
             topic_project=topic_project,
             custom_attributes=custom_attributes,
             event_types=event_types,
             blob_name_prefix=blob_name_prefix,
             payload_format=payload_format,
+            notification_id=notification_id,
         )
 
     def exists(self, client=None, timeout=_DEFAULT_TIMEOUT):
@@ -647,10 +649,10 @@ class Bucket(_PropertyMixin):
 
         :type client: :class:`~google.cloud.storage.client.Client` or
                       ``NoneType``
-        :param client: Optional. The client to use.  If not passed, falls back
+        :param client: (Optional) The client to use.  If not passed, falls back
                        to the ``client`` stored on the current bucket.
         :type timeout: float or tuple
-        :param timeout: (optional) The amount of time, in seconds, to wait
+        :param timeout: (Optional) The amount of time, in seconds, to wait
             for the server response.
 
             Can also be passed as a tuple (connect_timeout, read_timeout).
@@ -693,7 +695,7 @@ class Bucket(_PropertyMixin):
         predefined_default_object_acl=None,
         timeout=_DEFAULT_TIMEOUT,
     ):
-        """Creates current bucket.
+        """DEPRECATED. Creates current bucket.
 
         If the bucket already exists, will raise
         :class:`google.cloud.exceptions.Conflict`.
@@ -704,11 +706,11 @@ class Bucket(_PropertyMixin):
 
         :type client: :class:`~google.cloud.storage.client.Client` or
                       ``NoneType``
-        :param client: Optional. The client to use. If not passed, falls back
+        :param client: (Optional) The client to use. If not passed, falls back
                        to the ``client`` stored on the current bucket.
 
         :type project: str
-        :param project: Optional. The project under which the bucket is to
+        :param project: (Optional) The project under which the bucket is to
                         be created. If not passed, uses the project set on
                         the client.
         :raises ValueError: if :attr:`user_project` is set.
@@ -716,65 +718,45 @@ class Bucket(_PropertyMixin):
                             :attr:`project` is also None.
 
         :type location: str
-        :param location: Optional. The location of the bucket. If not passed,
+        :param location: (Optional) The location of the bucket. If not passed,
                          the default location, US, will be used. See
                          https://cloud.google.com/storage/docs/bucket-locations
 
         :type predefined_acl: str
         :param predefined_acl:
-            Optional. Name of predefined ACL to apply to bucket. See:
+            (Optional) Name of predefined ACL to apply to bucket. See:
             https://cloud.google.com/storage/docs/access-control/lists#predefined-acl
 
         :type predefined_default_object_acl: str
         :param predefined_default_object_acl:
-            Optional. Name of predefined ACL to apply to bucket's objects. See:
+            (Optional) Name of predefined ACL to apply to bucket's objects. See:
             https://cloud.google.com/storage/docs/access-control/lists#predefined-acl
 
         :type timeout: float or tuple
-        :param timeout: (optional) The amount of time, in seconds, to wait
+        :param timeout: (Optional) The amount of time, in seconds, to wait
             for the server response.
 
             Can also be passed as a tuple (connect_timeout, read_timeout).
             See :meth:`requests.Session.request` documentation for details.
         """
+        warnings.warn(
+            "Bucket.create() is deprecated and will be removed in future."
+            "Use Client.create_bucket() instead.",
+            PendingDeprecationWarning,
+            stacklevel=1,
+        )
         if self.user_project is not None:
             raise ValueError("Cannot create bucket with 'user_project' set.")
 
         client = self._require_client(client)
-
-        if project is None:
-            project = client.project
-
-        if project is None:
-            raise ValueError("Client project not set:  pass an explicit project.")
-
-        query_params = {"project": project}
-
-        if predefined_acl is not None:
-            predefined_acl = BucketACL.validate_predefined(predefined_acl)
-            query_params["predefinedAcl"] = predefined_acl
-
-        if predefined_default_object_acl is not None:
-            predefined_default_object_acl = DefaultObjectACL.validate_predefined(
-                predefined_default_object_acl
-            )
-            query_params["predefinedDefaultObjectAcl"] = predefined_default_object_acl
-
-        properties = {key: self._properties[key] for key in self._changes}
-        properties["name"] = self.name
-
-        if location is not None:
-            properties["location"] = location
-
-        api_response = client._connection.api_request(
-            method="POST",
-            path="/b",
-            query_params=query_params,
-            data=properties,
-            _target_object=self,
+        client.create_bucket(
+            bucket_or_name=self,
+            project=project,
+            location=location,
+            predefined_acl=predefined_acl,
+            predefined_default_object_acl=predefined_default_object_acl,
             timeout=timeout,
         )
-        self._set_properties(api_response)
 
     def patch(self, client=None, timeout=_DEFAULT_TIMEOUT):
         """Sends all changed properties in a PATCH request.
@@ -788,7 +770,7 @@ class Bucket(_PropertyMixin):
         :param client: the client to use.  If not passed, falls back to the
                        ``client`` stored on the current object.
         :type timeout: float or tuple
-        :param timeout: (optional) The amount of time, in seconds, to wait
+        :param timeout: (Optional) The amount of time, in seconds, to wait
             for the server response.
 
             Can also be passed as a tuple (connect_timeout, read_timeout).
@@ -859,21 +841,21 @@ class Bucket(_PropertyMixin):
 
         :type client: :class:`~google.cloud.storage.client.Client` or
                       ``NoneType``
-        :param client: Optional. The client to use.  If not passed, falls back
+        :param client: (Optional) The client to use.  If not passed, falls back
                        to the ``client`` stored on the current bucket.
 
         :type encryption_key: bytes
         :param encryption_key:
-            Optional 32 byte encryption key for customer-supplied encryption.
+            (Optional) 32 byte encryption key for customer-supplied encryption.
             See
             https://cloud.google.com/storage/docs/encryption#customer-supplied.
 
         :type generation: long
-        :param generation: Optional. If present, selects a specific revision of
+        :param generation: (Optional) If present, selects a specific revision of
                            this object.
 
         :type timeout: float or tuple
-        :param timeout: (optional) The amount of time, in seconds, to wait
+        :param timeout: (Optional) The amount of time, in seconds, to wait
             for the server response.
 
             Can also be passed as a tuple (connect_timeout, read_timeout).
@@ -934,7 +916,7 @@ class Bucket(_PropertyMixin):
             token.
 
         :type prefix: str
-        :param prefix: (Optional) prefix used to filter blobs.
+        :param prefix: (Optional) Prefix used to filter blobs.
 
         :type delimiter: str
         :param delimiter: (Optional) Delimiter, used with ``prefix`` to
@@ -963,7 +945,7 @@ class Bucket(_PropertyMixin):
                        to the ``client`` stored on the current bucket.
 
         :type timeout: float or tuple
-        :param timeout: (optional) The amount of time, in seconds, to wait
+        :param timeout: (Optional) The amount of time, in seconds, to wait
             for the server response.
 
             Can also be passed as a tuple (connect_timeout, read_timeout).
@@ -1017,10 +999,10 @@ class Bucket(_PropertyMixin):
 
         :type client: :class:`~google.cloud.storage.client.Client` or
                       ``NoneType``
-        :param client: Optional. The client to use.  If not passed, falls back
+        :param client: (Optional) The client to use.  If not passed, falls back
                        to the ``client`` stored on the current bucket.
         :type timeout: float or tuple
-        :param timeout: (optional) The amount of time, in seconds, to wait
+        :param timeout: (Optional) The amount of time, in seconds, to wait
             for the server response.
 
             Can also be passed as a tuple (connect_timeout, read_timeout).
@@ -1041,6 +1023,44 @@ class Bucket(_PropertyMixin):
         iterator.bucket = self
         return iterator
 
+    def get_notification(self, notification_id, client=None, timeout=_DEFAULT_TIMEOUT):
+        """Get Pub / Sub notification for this bucket.
+
+        See:
+        https://cloud.google.com/storage/docs/json_api/v1/notifications/get
+
+        If :attr:`user_project` is set, bills the API request to that project.
+
+        :type notification_id: str
+        :param notification_id: The notification id to retrieve the notification configuration.
+
+        :type client: :class:`~google.cloud.storage.client.Client` or
+                      ``NoneType``
+        :param client: (Optional) The client to use.  If not passed, falls back
+                       to the ``client`` stored on the current bucket.
+        :type timeout: float or tuple
+        :param timeout: (Optional) The amount of time, in seconds, to wait
+            for the server response.
+
+            Can also be passed as a tuple (connect_timeout, read_timeout).
+            See :meth:`requests.Session.request` documentation for details.
+
+        :rtype: :class:`.BucketNotification`
+        :returns: notification instance.
+
+        Example:
+            Get notification using notification id.
+
+            >>> from google.cloud import storage
+            >>> client = storage.Client()
+            >>> bucket = client.get_bucket('my-bucket-name')  # API request.
+            >>> notification = bucket.get_notification(notification_id='id')  # API request.
+
+        """
+        notification = self.notification(notification_id=notification_id)
+        notification.reload(client=client, timeout=timeout)
+        return notification
+
     def delete(self, force=False, client=None, timeout=_DEFAULT_TIMEOUT):
         """Delete this bucket.
 
@@ -1065,10 +1085,10 @@ class Bucket(_PropertyMixin):
 
         :type client: :class:`~google.cloud.storage.client.Client` or
                       ``NoneType``
-        :param client: Optional. The client to use.  If not passed, falls back
+        :param client: (Optional) The client to use.  If not passed, falls back
                        to the ``client`` stored on the current bucket.
         :type timeout: float or tuple
-        :param timeout: (optional) The amount of time, in seconds, to wait
+        :param timeout: (Optional) The amount of time, in seconds, to wait
             for the server response on each request.
 
             Can also be passed as a tuple (connect_timeout, read_timeout).
@@ -1137,15 +1157,15 @@ class Bucket(_PropertyMixin):
 
         :type client: :class:`~google.cloud.storage.client.Client` or
                       ``NoneType``
-        :param client: Optional. The client to use.  If not passed, falls back
+        :param client: (Optional) The client to use.  If not passed, falls back
                        to the ``client`` stored on the current bucket.
 
         :type generation: long
-        :param generation: Optional. If present, permanently deletes a specific
+        :param generation: (Optional) If present, permanently deletes a specific
                            revision of this object.
 
         :type timeout: float or tuple
-        :param timeout: (optional) The amount of time, in seconds, to wait
+        :param timeout: (Optional) The amount of time, in seconds, to wait
             for the server response.
 
             Can also be passed as a tuple (connect_timeout, read_timeout).
@@ -1196,7 +1216,7 @@ class Bucket(_PropertyMixin):
                        to the ``client`` stored on the current bucket.
 
         :type timeout: float or tuple
-        :param timeout: (optional) The amount of time, in seconds, to wait
+        :param timeout: (Optional) The amount of time, in seconds, to wait
             for the server response. The timeout applies to each individual
             blob delete request.
 
@@ -1240,23 +1260,23 @@ class Bucket(_PropertyMixin):
                                    copied.
 
         :type new_name: str
-        :param new_name: (optional) the new name for the copied file.
+        :param new_name: (Optional) The new name for the copied file.
 
         :type client: :class:`~google.cloud.storage.client.Client` or
                       ``NoneType``
-        :param client: Optional. The client to use.  If not passed, falls back
+        :param client: (Optional) The client to use.  If not passed, falls back
                        to the ``client`` stored on the current bucket.
 
         :type preserve_acl: bool
-        :param preserve_acl: Optional. Copies ACL from old blob to new blob.
+        :param preserve_acl: (Optional) Copies ACL from old blob to new blob.
                              Default: True.
 
         :type source_generation: long
-        :param source_generation: Optional. The generation of the blob to be
+        :param source_generation: (Optional) The generation of the blob to be
                                   copied.
 
         :type timeout: float or tuple
-        :param timeout: (optional) The amount of time, in seconds, to wait
+        :param timeout: (Optional) The amount of time, in seconds, to wait
             for the server response.
 
             Can also be passed as a tuple (connect_timeout, read_timeout).
@@ -1315,11 +1335,11 @@ class Bucket(_PropertyMixin):
 
         :type client: :class:`~google.cloud.storage.client.Client` or
                       ``NoneType``
-        :param client: Optional. The client to use.  If not passed, falls back
+        :param client: (Optional) The client to use.  If not passed, falls back
                        to the ``client`` stored on the current bucket.
 
         :type timeout: float or tuple
-        :param timeout: (optional) The amount of time, in seconds, to wait
+        :param timeout: (Optional) The amount of time, in seconds, to wait
             for the server response. The timeout applies to each individual
             request.
 
@@ -1908,7 +1928,7 @@ class Bucket(_PropertyMixin):
     def requester_pays(self, value):
         """Update whether requester pays for API requests for this bucket.
 
-        See  https://cloud.google.com/storage/docs/<DOCS-MISSING> for
+        See https://cloud.google.com/storage/docs/using-requester-pays for
         details.
 
         :type value: convertible to boolean
@@ -1974,11 +1994,11 @@ class Bucket(_PropertyMixin):
 
         :type client: :class:`~google.cloud.storage.client.Client` or
                       ``NoneType``
-        :param client: Optional. The client to use.  If not passed, falls back
+        :param client: (Optional) The client to use.  If not passed, falls back
                        to the ``client`` stored on the current bucket.
 
         :type requested_policy_version: int or ``NoneType``
-        :param requested_policy_version: Optional. The version of IAM policies to request.
+        :param requested_policy_version: (Optional) The version of IAM policies to request.
                                          If a policy with a condition is requested without
                                          setting this, the server will return an error.
                                          This must be set to a value of 3 to retrieve IAM
@@ -1990,7 +2010,7 @@ class Bucket(_PropertyMixin):
                                          feature syntax in the policy fetched.
 
         :type timeout: float or tuple
-        :param timeout: (optional) The amount of time, in seconds, to wait
+        :param timeout: (Optional) The amount of time, in seconds, to wait
             for the server response.
 
             Can also be passed as a tuple (connect_timeout, read_timeout).
@@ -2055,11 +2075,11 @@ class Bucket(_PropertyMixin):
 
         :type client: :class:`~google.cloud.storage.client.Client` or
                       ``NoneType``
-        :param client: Optional. The client to use.  If not passed, falls back
+        :param client: (Optional) The client to use.  If not passed, falls back
                        to the ``client`` stored on the current bucket.
 
         :type timeout: float or tuple
-        :param timeout: (optional) The amount of time, in seconds, to wait
+        :param timeout: (Optional) The amount of time, in seconds, to wait
             for the server response.
 
             Can also be passed as a tuple (connect_timeout, read_timeout).
@@ -2100,11 +2120,11 @@ class Bucket(_PropertyMixin):
 
         :type client: :class:`~google.cloud.storage.client.Client` or
                       ``NoneType``
-        :param client: Optional. The client to use.  If not passed, falls back
+        :param client: (Optional) The client to use.  If not passed, falls back
                        to the ``client`` stored on the current bucket.
 
         :type timeout: float or tuple
-        :param timeout: (optional) The amount of time, in seconds, to wait
+        :param timeout: (Optional) The amount of time, in seconds, to wait
             for the server response.
 
             Can also be passed as a tuple (connect_timeout, read_timeout).
@@ -2141,10 +2161,10 @@ class Bucket(_PropertyMixin):
 
         :type client: :class:`~google.cloud.storage.client.Client` or
                       ``NoneType``
-        :param client: Optional. The client to use.  If not passed, falls back
+        :param client: (Optional) The client to use.  If not passed, falls back
                        to the ``client`` stored on the current bucket.
         :type timeout: float or tuple
-        :param timeout: (optional) The amount of time, in seconds, to wait
+        :param timeout: (Optional) The amount of time, in seconds, to wait
             for the server response. The timeout applies to each underlying
             request.
 
@@ -2207,11 +2227,11 @@ class Bucket(_PropertyMixin):
 
         :type client: :class:`~google.cloud.storage.client.Client` or
                       ``NoneType``
-        :param client: Optional. The client to use.  If not passed, falls back
+        :param client: (Optional) The client to use.  If not passed, falls back
                        to the ``client`` stored on the current bucket.
 
         :type timeout: float or tuple
-        :param timeout: (optional) The amount of time, in seconds, to wait
+        :param timeout: (Optional) The amount of time, in seconds, to wait
             for the server response. The timeout applies to each underlying
             request.
 
@@ -2277,7 +2297,7 @@ class Bucket(_PropertyMixin):
             /post-object#policydocument
 
         :type expiration: datetime
-        :param expiration: Optional expiration in UTC. If not specified, the
+        :param expiration: (Optional) Expiration in UTC. If not specified, the
                            policy will expire in 1 hour.
 
         :type conditions: list
@@ -2285,7 +2305,7 @@ class Bucket(_PropertyMixin):
                           `policy documents`_ documentation.
 
         :type client: :class:`~google.cloud.storage.client.Client`
-        :param client: Optional. The client to use.  If not passed, falls back
+        :param client: (Optional) The client to use.  If not passed, falls back
                        to the ``client`` stored on the current bucket.
 
         :rtype: dict
@@ -2325,7 +2345,7 @@ class Bucket(_PropertyMixin):
         """Lock the bucket's retention policy.
 
         :type timeout: float or tuple
-        :param timeout: (optional) The amount of time, in seconds, to wait
+        :param timeout: (Optional) The amount of time, in seconds, to wait
             for the server response.
 
             Can also be passed as a tuple (connect_timeout, read_timeout).
@@ -2374,6 +2394,9 @@ class Bucket(_PropertyMixin):
         client=None,
         credentials=None,
         version=None,
+        virtual_hosted_style=False,
+        bucket_bound_hostname=None,
+        scheme="http",
     ):
         """Generates a signed URL for this bucket.
 
@@ -2392,6 +2415,20 @@ class Bucket(_PropertyMixin):
         amount of time, you can use this method to generate a URL that
         is only valid within a certain time period.
 
+        If ``bucket_bound_hostname`` is set as an argument of :attr:`api_access_endpoint`,
+        ``https`` works only if using a ``CDN``.
+
+        Example:
+            Generates a signed URL for this bucket using bucket_bound_hostname and scheme.
+
+            >>> from google.cloud import storage
+            >>> client = storage.Client()
+            >>> bucket = client.get_bucket('my-bucket-name')
+            >>> url = bucket.generate_signed_url(expiration='url-expiration-time', bucket_bound_hostname='mydomain.tld',
+            >>>                                  version='v4')
+            >>> url = bucket.generate_signed_url(expiration='url-expiration-time', bucket_bound_hostname='mydomain.tld',
+            >>>                                  version='v4',scheme='https')  # If using ``CDN``
+
         This is particularly useful if you don't want publicly
         accessible buckets, but don't want to require users to explicitly
         log in.
@@ -2400,7 +2437,7 @@ class Bucket(_PropertyMixin):
         :param expiration: Point in time when the signed URL should expire.
 
         :type api_access_endpoint: str
-        :param api_access_endpoint: Optional URI base.
+        :param api_access_endpoint: (Optional) URI base.
 
         :type method: str
         :param method: The HTTP verb that will be used when requesting the URL.
@@ -2415,7 +2452,7 @@ class Bucket(_PropertyMixin):
 
         :type query_parameters: dict
         :param query_parameters:
-            (Optional) Additional query paramtersto be included as part of the
+            (Optional) Additional query parameters to be included as part of the
             signed URLs.  See:
             https://cloud.google.com/storage/docs/xml-api/reference-headers#query
 
@@ -2436,6 +2473,23 @@ class Bucket(_PropertyMixin):
         :param version: (Optional) The version of signed credential to create.
                         Must be one of 'v2' | 'v4'.
 
+        :type virtual_hosted_style: bool
+        :param virtual_hosted_style:
+            (Optional) If true, then construct the URL relative the bucket's
+            virtual hostname, e.g., '<bucket-name>.storage.googleapis.com'.
+
+        :type bucket_bound_hostname: str
+        :param bucket_bound_hostname:
+            (Optional) If pass, then construct the URL relative to the bucket-bound hostname.
+            Value cane be a bare or with scheme, e.g., 'example.com' or 'http://example.com'.
+            See: https://cloud.google.com/storage/docs/request-endpoints#cname
+
+        :type scheme: str
+        :param scheme:
+            (Optional) If ``bucket_bound_hostname`` is passed as a bare hostname, use
+            this value as the scheme.  ``https`` will work only when using a CDN.
+            Defaults to ``"http"``.
+
         :raises: :exc:`ValueError` when version is invalid.
         :raises: :exc:`TypeError` when expiration is not a valid type.
         :raises: :exc:`AttributeError` if credentials is not an instance
@@ -2450,7 +2504,22 @@ class Bucket(_PropertyMixin):
         elif version not in ("v2", "v4"):
             raise ValueError("'version' must be either 'v2' or 'v4'")
 
-        resource = "/{bucket_name}".format(bucket_name=self.name)
+        if virtual_hosted_style:
+            api_access_endpoint = "https://{bucket_name}.storage.googleapis.com".format(
+                bucket_name=self.name
+            )
+        elif bucket_bound_hostname:
+            if ":" in bucket_bound_hostname:
+                api_access_endpoint = bucket_bound_hostname
+            else:
+                api_access_endpoint = "{scheme}://{bucket_bound_hostname}".format(
+                    scheme=scheme, bucket_bound_hostname=bucket_bound_hostname
+                )
+        else:
+            resource = "/{bucket_name}".format(bucket_name=self.name)
+
+        if virtual_hosted_style or bucket_bound_hostname:
+            resource = "/"
 
         if credentials is None:
             client = self._require_client(client)
