@@ -14,18 +14,29 @@
 
 """Client for interacting with the Google Cloud Storage API."""
 
+import base64
+import binascii
+import collections
+import datetime
 import functools
-
+import json
+import warnings
 import google.api_core.client_options
 
 from google.auth.credentials import AnonymousCredentials
 
 from google.api_core import page_iterator
-from google.cloud._helpers import _LocalStack
+from google.cloud._helpers import _LocalStack, _NOW
 from google.cloud.client import ClientWithProject
 from google.cloud.exceptions import NotFound
 from google.cloud.storage._helpers import _get_storage_host
 from google.cloud.storage._http import Connection
+from google.cloud.storage._signing import (
+    get_expiration_seconds_v4,
+    get_v4_now_dtstamps,
+    ensure_signed_credentials,
+    _sign_message,
+)
 from google.cloud.storage.batch import Batch
 from google.cloud.storage.bucket import Bucket
 from google.cloud.storage.blob import Blob
@@ -222,7 +233,7 @@ class Client(ClientWithProject):
             (Optional) Project ID to use for retreiving GCS service account
             email address.  Defaults to the client's project.
         :type timeout: float or tuple
-        :param timeout: (optional) The amount of time, in seconds, to wait
+        :param timeout: (Optional) The amount of time, in seconds, to wait
             for the server response.
 
             Can also be passed as a tuple (connect_timeout, read_timeout).
@@ -250,7 +261,7 @@ class Client(ClientWithProject):
         :param bucket_name: The name of the bucket to be instantiated.
 
         :type user_project: str
-        :param user_project: (Optional) the project ID to be billed for API
+        :param user_project: (Optional) The project ID to be billed for API
                              requests made via the bucket.
 
         :rtype: :class:`google.cloud.storage.bucket.Bucket`
@@ -336,7 +347,7 @@ class Client(ClientWithProject):
         :param bucket_name: The name of the bucket to get.
 
         :type timeout: float or tuple
-        :param timeout: (optional) The amount of time, in seconds, to wait
+        :param timeout: (Optional) The amount of time, in seconds, to wait
             for the server response.
 
             Can also be passed as a tuple (connect_timeout, read_timeout).
@@ -373,23 +384,24 @@ class Client(ClientWithProject):
             ]):
                 The bucket resource to pass or name to create.
             requester_pays (bool):
-                Optional. Whether requester pays for API requests for this
-                bucket and its blobs.
+                DEPRECATED. Use Bucket().requester_pays instead.
+                (Optional) Whether requester pays for API requests for
+                this bucket and its blobs.
             project (str):
-                Optional. The project under which the bucket is to be created.
+                (Optional) The project under which the bucket is to be created.
                 If not passed, uses the project set on the client.
             user_project (str):
-                Optional. The project ID to be billed for API requests
+                (Optional) The project ID to be billed for API requests
                 made via created bucket.
             location (str):
-                Optional. The location of the bucket. If not passed,
+                (Optional) The location of the bucket. If not passed,
                 the default location, US, will be used. See
                 https://cloud.google.com/storage/docs/bucket-locations
             predefined_acl (str):
-                Optional. Name of predefined ACL to apply to bucket. See:
+                (Optional) Name of predefined ACL to apply to bucket. See:
                 https://cloud.google.com/storage/docs/access-control/lists#predefined-acl
             predefined_default_object_acl (str):
-                Optional. Name of predefined ACL to apply to bucket's objects. See:
+                (Optional) Name of predefined ACL to apply to bucket's objects. See:
                 https://cloud.google.com/storage/docs/access-control/lists#predefined-acl
             timeout (Optional[Union[float, Tuple[float, float]]]):
                 The amount of time, in seconds, to wait for the server response.
@@ -435,6 +447,11 @@ class Client(ClientWithProject):
             raise ValueError("Client project not set:  pass an explicit project.")
 
         if requester_pays is not None:
+            warnings.warn(
+                "requester_pays arg is deprecated. Use Bucket().requester_pays instead.",
+                PendingDeprecationWarning,
+                stacklevel=1,
+            )
             bucket.requester_pays = requester_pays
 
         query_params = {"project": project}
@@ -482,9 +499,9 @@ class Client(ClientWithProject):
             file_obj (file):
                 A file handle to which to write the blob's data.
             start (int):
-                Optional. The first byte in a range to be downloaded.
+                (Optional) The first byte in a range to be downloaded.
             end (int):
-                Optional. The last byte in a range to be downloaded.
+                (Optional) The last byte in a range to be downloaded.
 
         Examples:
             Download a blob using using a blob resource.
@@ -550,7 +567,7 @@ class Client(ClientWithProject):
                 token.
 
             prefix (str):
-                (Optional) prefix used to filter blobs.
+                (Optional) Prefix used to filter blobs.
 
             delimiter (str):
                 (Optional) Delimiter, used with ``prefix`` to
@@ -618,18 +635,18 @@ class Client(ClientWithProject):
         This implements "storage.buckets.list".
 
         :type max_results: int
-        :param max_results: Optional. The maximum number of buckets to return.
+        :param max_results: (Optional) The maximum number of buckets to return.
 
         :type page_token: str
         :param page_token:
-            Optional. If present, return the next batch of buckets, using the
+            (Optional) If present, return the next batch of buckets, using the
             value, which must correspond to the ``nextPageToken`` value
             returned in the previous response.  Deprecated: use the ``pages``
             property of the returned iterator instead of manually passing the
             token.
 
         :type prefix: str
-        :param prefix: Optional. Filter results to buckets whose names begin
+        :param prefix: (Optional) Filter results to buckets whose names begin
                        with this prefix.
 
         :type projection: str
@@ -645,11 +662,11 @@ class Client(ClientWithProject):
             bucket returned: 'items/id,nextPageToken'
 
         :type project: str
-        :param project: (Optional) the project whose buckets are to be listed.
+        :param project: (Optional) The project whose buckets are to be listed.
                         If not passed, uses the project set on the client.
 
         :type timeout: float or tuple
-        :param timeout: (optional) The amount of time, in seconds, to wait
+        :param timeout: (Optional) The amount of time, in seconds, to wait
             for the server response.
 
             Can also be passed as a tuple (connect_timeout, read_timeout).
@@ -702,14 +719,14 @@ class Client(ClientWithProject):
         :param service_account_email: e-mail address of the service account
 
         :type project_id: str
-        :param project_id: (Optional) explicit project ID for the key.
+        :param project_id: (Optional) Explicit project ID for the key.
             Defaults to the client's project.
 
         :type user_project: str
         :param user_project: (Optional) This parameter is currently ignored.
 
         :type timeout: float or tuple
-        :param timeout: (optional) The amount of time, in seconds, to wait
+        :param timeout: (Optional) The amount of time, in seconds, to wait
             for the server response.
 
             Can also be passed as a tuple (connect_timeout, read_timeout).
@@ -749,26 +766,26 @@ class Client(ClientWithProject):
 
         :type max_results: int
         :param max_results:
-            (Optional) max number of keys to return in a given page.
+            (Optional) Max number of keys to return in a given page.
 
         :type service_account_email: str
         :param service_account_email:
-            (Optional) limit keys to those created by the given service account.
+            (Optional) Limit keys to those created by the given service account.
 
         :type show_deleted_keys: bool
         :param show_deleted_keys:
-            (Optional) included deleted keys in the list. Default is to
+            (Optional) Included deleted keys in the list. Default is to
             exclude them.
 
         :type project_id: str
-        :param project_id: (Optional) explicit project ID for the key.
+        :param project_id: (Optional) Explicit project ID for the key.
             Defaults to the client's project.
 
         :type user_project: str
         :param user_project: (Optional) This parameter is currently ignored.
 
         :type timeout: float or tuple
-        :param timeout: (optional) The amount of time, in seconds, to wait
+        :param timeout: (Optional) The amount of time, in seconds, to wait
             for the server response.
 
             Can also be passed as a tuple (connect_timeout, read_timeout).
@@ -813,11 +830,11 @@ class Client(ClientWithProject):
         :param access_id: Unique ID of an existing key.
 
         :type project_id: str
-        :param project_id: (Optional) project ID of an existing key.
+        :param project_id: (Optional) Project ID of an existing key.
             Defaults to client's project.
 
         :type timeout: float or tuple
-        :param timeout: (optional) The amount of time, in seconds, to wait
+        :param timeout: (Optional) The amount of time, in seconds, to wait
             for the server response.
 
             Can also be passed as a tuple (connect_timeout, read_timeout).
@@ -830,6 +847,181 @@ class Client(ClientWithProject):
         metadata.reload(timeout=timeout)  # raises NotFound for missing key
         return metadata
 
+    def generate_signed_post_policy_v4(
+        self,
+        bucket_name,
+        blob_name,
+        expiration,
+        conditions=None,
+        fields=None,
+        credentials=None,
+        virtual_hosted_style=False,
+        bucket_bound_hostname=None,
+        scheme=None,
+        service_account_email=None,
+        access_token=None,
+    ):
+        """Generate a V4 signed policy object.
+
+        .. note::
+
+            Assumes ``credentials`` implements the
+            :class:`google.auth.credentials.Signing` interface. Also assumes
+            ``credentials`` has a ``service_account_email`` property which
+            identifies the credentials.
+
+        Generated policy object allows user to upload objects with a POST request.
+
+        :type bucket_name: str
+        :param bucket_name: Bucket name.
+
+        :type blob_name: str
+        :param blob_name: Object name.
+
+        :type expiration: Union[Integer, datetime.datetime, datetime.timedelta]
+        :param expiration: Policy expiration time.
+
+        :type conditions: list
+        :param conditions: (Optional) List of POST policy conditions, which are
+                           used to restrict what is allowed in the request.
+
+        :type fields: dict
+        :param fields: (Optional) Additional elements to include into request.
+
+        :type credentials: :class:`google.auth.credentials.Signing`
+        :param credentials: (Optional) Credentials object with an associated private
+                            key to sign text.
+
+        :type virtual_hosted_style: bool
+        :param virtual_hosted_style: (Optional) If True, construct the URL relative to the bucket
+                                     virtual hostname, e.g., '<bucket-name>.storage.googleapis.com'.
+
+        :type bucket_bound_hostname: str
+        :param bucket_bound_hostname:
+            (Optional) If passed, construct the URL relative to the bucket-bound hostname.
+            Value can be bare or with a scheme, e.g., 'example.com' or 'http://example.com'.
+            See: https://cloud.google.com/storage/docs/request-endpoints#cname
+
+        :type scheme: str
+        :param scheme:
+            (Optional) If ``bucket_bound_hostname`` is passed as a bare hostname, use
+            this value as a scheme. ``https`` will work only when using a CDN.
+            Defaults to ``"http"``.
+
+        :type service_account_email: str
+        :param service_account_email: (Optional) E-mail address of the service account.
+
+        :type access_token: str
+        :param access_token: (Optional) Access token for a service account.
+
+        :rtype: dict
+        :returns: Signed POST policy.
+
+        Example:
+            Generate signed POST policy and upload a file.
+
+            >>> from google.cloud import storage
+            >>> client = storage.Client()
+            >>> policy = client.generate_signed_post_policy_v4(
+                "bucket-name",
+                "blob-name",
+                expiration=datetime.datetime(2020, 3, 17),
+                conditions=[
+                    ["content-length-range", 0, 255]
+                ],
+                fields=[
+                    "x-goog-meta-hello" => "world"
+                ],
+            )
+            >>> with open("bucket-name", "rb") as f:
+                files = {"file": ("bucket-name", f)}
+                requests.post(policy["url"], data=policy["fields"], files=files)
+        """
+        credentials = self._credentials if credentials is None else credentials
+        ensure_signed_credentials(credentials)
+
+        # prepare policy conditions and fields
+        timestamp, datestamp = get_v4_now_dtstamps()
+
+        x_goog_credential = "{email}/{datestamp}/auto/storage/goog4_request".format(
+            email=credentials.signer_email, datestamp=datestamp
+        )
+        required_conditions = [
+            {"key": blob_name},
+            {"x-goog-date": timestamp},
+            {"x-goog-credential": x_goog_credential},
+            {"x-goog-algorithm": "GOOG4-RSA-SHA256"},
+        ]
+
+        conditions = conditions or []
+        policy_fields = {}
+        for key, value in sorted((fields or {}).items()):
+            if not key.startswith("x-ignore-"):
+                policy_fields[key] = value
+                conditions.append({key: value})
+
+        conditions += required_conditions
+
+        # calculate policy expiration time
+        now = _NOW()
+        if expiration is None:
+            expiration = now + datetime.timedelta(hours=1)
+
+        policy_expires = now + datetime.timedelta(
+            seconds=get_expiration_seconds_v4(expiration)
+        )
+
+        # encode policy for signing
+        policy = json.dumps(
+            collections.OrderedDict(
+                sorted(
+                    {
+                        "conditions": conditions,
+                        "expiration": policy_expires.isoformat() + "Z",
+                    }.items()
+                )
+            ),
+            separators=(",", ":"),
+        )
+        str_to_sign = base64.b64encode(policy.encode("utf-8"))
+
+        # sign the policy and get its cryptographic signature
+        if access_token and service_account_email:
+            signature = _sign_message(str_to_sign, access_token, service_account_email)
+            signature_bytes = base64.b64decode(signature)
+        else:
+            signature_bytes = credentials.sign_bytes(str_to_sign)
+
+        # get hexadecimal representation of the signature
+        signature = binascii.hexlify(signature_bytes).decode("utf-8")
+
+        policy_fields.update(
+            {
+                "key": blob_name,
+                "x-goog-algorithm": "GOOG4-RSA-SHA256",
+                "x-goog-credential": x_goog_credential,
+                "x-goog-date": timestamp,
+                "x-goog-signature": signature,
+                "policy": str_to_sign,
+            }
+        )
+        # designate URL
+        if virtual_hosted_style:
+            url = "https://{}.storage.googleapis.com/".format(bucket_name)
+
+        elif bucket_bound_hostname:
+            if ":" in bucket_bound_hostname:  # URL includes scheme
+                url = bucket_bound_hostname
+
+            else:  # scheme is given separately
+                url = "{scheme}://{host}/".format(
+                    scheme=scheme, host=bucket_bound_hostname
+                )
+        else:
+            url = "https://storage.googleapis.com/{}/".format(bucket_name)
+
+        return {"url": url, "fields": policy_fields}
+
 
 def _item_to_bucket(iterator, item):
     """Convert a JSON bucket to the native object.
