@@ -1,3 +1,5 @@
+# -*- coding: utf-8 -*-
+#
 # Copyright 2017 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -16,9 +18,7 @@ import base64
 import binascii
 import calendar
 import datetime
-import io
 import json
-import os
 import time
 import unittest
 
@@ -27,12 +27,7 @@ import pytest
 import six
 from six.moves import urllib_parse
 
-
-def _read_local_json(json_file):
-    here = os.path.dirname(__file__)
-    json_path = os.path.abspath(os.path.join(here, json_file))
-    with io.open(json_path, "r", encoding="utf-8-sig") as fileobj:
-        return json.load(fileobj)
+from . import _read_local_json
 
 
 _SERVICE_ACCOUNT_JSON = _read_local_json("url_signer_v4_test_account.json")
@@ -309,12 +304,12 @@ class Test_get_canonical_headers(unittest.TestCase):
         self.assertEqual(ordered, expected_ordered)
 
 
-class Test_canonicalize(unittest.TestCase):
+class Test_canonicalize_v2(unittest.TestCase):
     @staticmethod
     def _call_fut(*args, **kwargs):
-        from google.cloud.storage._signing import canonicalize
+        from google.cloud.storage._signing import canonicalize_v2
 
-        return canonicalize(*args, **kwargs)
+        return canonicalize_v2(*args, **kwargs)
 
     def test_wo_headers_or_query_parameters(self):
         method = "GET"
@@ -337,7 +332,7 @@ class Test_canonicalize(unittest.TestCase):
             canonical.headers, ["x-goog-extension:foobar", "x-goog-resumable:start"]
         )
 
-    def test_w_query_paramters(self):
+    def test_w_query_parameters(self):
         method = "GET"
         resource = "/bucket/blob"
         query_parameters = {"foo": "bar", "baz": "qux"}
@@ -650,6 +645,9 @@ class Test_generate_signed_url_v4(unittest.TestCase):
     def test_w_custom_headers(self):
         self._generate_helper(headers={"x-goog-foo": "bar"})
 
+    def test_w_custom_payload_hash_goog(self):
+        self._generate_helper(headers={"x-goog-content-sha256": "DEADBEEF"})
+
     def test_w_custom_query_parameters_w_string_value(self):
         self._generate_helper(query_parameters={"bar": "/"})
 
@@ -702,6 +700,77 @@ class Test_sign_message(unittest.TestCase):
                 )
 
 
+class TestCustomURLEncoding(unittest.TestCase):
+    def test_url_encode(self):
+        from google.cloud.storage._signing import _url_encode
+
+        # param1 includes safe symbol ~
+        # param# includes symbols, which must be encoded
+        query_params = {"param1": "value~1-2", "param#": "*value+value/"}
+
+        self.assertEqual(
+            _url_encode(query_params), "param%23=%2Avalue%2Bvalue%2F&param1=value~1-2"
+        )
+
+
+class TestQuoteParam(unittest.TestCase):
+    def test_ascii_symbols(self):
+        from google.cloud.storage._signing import _quote_param
+
+        encoded_param = _quote_param("param")
+        self.assertIsInstance(encoded_param, str)
+        self.assertEqual(encoded_param, "param")
+
+    def test_quoted_symbols(self):
+        from google.cloud.storage._signing import _quote_param
+
+        encoded_param = _quote_param("!#$%&'()*+,/:;=?@[]")
+        self.assertIsInstance(encoded_param, str)
+        self.assertEqual(
+            encoded_param, "%21%23%24%25%26%27%28%29%2A%2B%2C%2F%3A%3B%3D%3F%40%5B%5D"
+        )
+
+    def test_unquoted_symbols(self):
+        from google.cloud.storage._signing import _quote_param
+        import string
+
+        UNQUOTED = string.ascii_letters + string.digits + ".~_-"
+
+        encoded_param = _quote_param(UNQUOTED)
+        self.assertIsInstance(encoded_param, str)
+        self.assertEqual(encoded_param, UNQUOTED)
+
+    def test_unicode_symbols(self):
+        from google.cloud.storage._signing import _quote_param
+
+        encoded_param = _quote_param("ЁЙЦЯЩЯЩ")
+        self.assertIsInstance(encoded_param, str)
+        self.assertEqual(encoded_param, "%D0%81%D0%99%D0%A6%D0%AF%D0%A9%D0%AF%D0%A9")
+
+    def test_bytes(self):
+        from google.cloud.storage._signing import _quote_param
+
+        encoded_param = _quote_param(b"bytes")
+        self.assertIsInstance(encoded_param, str)
+        self.assertEqual(encoded_param, "bytes")
+
+
+class TestV4Stamps(unittest.TestCase):
+    def test_get_v4_now_dtstamps(self):
+        import datetime
+        from google.cloud.storage._signing import get_v4_now_dtstamps
+
+        with mock.patch(
+            "google.cloud.storage._signing.NOW",
+            return_value=datetime.datetime(2020, 3, 12, 13, 14, 15),
+        ) as now_mock:
+            timestamp, datestamp = get_v4_now_dtstamps()
+            now_mock.assert_called_once()
+
+        self.assertEqual(timestamp, "20200312T131415Z")
+        self.assertEqual(datestamp, "20200312")
+
+
 _DUMMY_SERVICE_ACCOUNT = None
 
 
@@ -718,16 +787,22 @@ def dummy_service_account():
     return _DUMMY_SERVICE_ACCOUNT
 
 
-def _run_conformance_test(resource, test_data):
-    credentials = dummy_service_account()
+_API_ACCESS_ENDPOINT = "https://storage.googleapis.com"
 
+
+def _run_conformance_test(
+    resource, test_data, api_access_endpoint=_API_ACCESS_ENDPOINT
+):
+    credentials = dummy_service_account()
     url = Test_generate_signed_url_v4._call_fut(
         credentials,
         resource,
         expiration=test_data["expiration"],
+        api_access_endpoint=api_access_endpoint,
         method=test_data["method"],
         _request_timestamp=test_data["timestamp"],
         headers=test_data.get("headers"),
+        query_parameters=test_data.get("queryParameters"),
     )
 
     assert url == test_data["expectedUrl"]
@@ -741,14 +816,39 @@ def test_conformance_client(test_data):
 
 @pytest.mark.parametrize("test_data", _BUCKET_TESTS)
 def test_conformance_bucket(test_data):
-    resource = "/{}".format(test_data["bucket"])
-    _run_conformance_test(resource, test_data)
+    global _API_ACCESS_ENDPOINT
+    if "urlStyle" in test_data and test_data["urlStyle"] == "BUCKET_BOUND_HOSTNAME":
+        _API_ACCESS_ENDPOINT = "{scheme}://{bucket_bound_hostname}".format(
+            scheme=test_data["scheme"],
+            bucket_bound_hostname=test_data["bucketBoundHostname"],
+        )
+        resource = "/"
+        _run_conformance_test(resource, test_data, _API_ACCESS_ENDPOINT)
+    else:
+        resource = "/{}".format(test_data["bucket"])
+        _run_conformance_test(resource, test_data)
 
 
 @pytest.mark.parametrize("test_data", _BLOB_TESTS)
 def test_conformance_blob(test_data):
-    resource = "/{}/{}".format(test_data["bucket"], test_data["object"])
-    _run_conformance_test(resource, test_data)
+    global _API_ACCESS_ENDPOINT
+    if "urlStyle" in test_data:
+        if test_data["urlStyle"] == "BUCKET_BOUND_HOSTNAME":
+            _API_ACCESS_ENDPOINT = "{scheme}://{bucket_bound_hostname}".format(
+                scheme=test_data["scheme"],
+                bucket_bound_hostname=test_data["bucketBoundHostname"],
+            )
+
+        # For the VIRTUAL_HOSTED_STYLE
+        else:
+            _API_ACCESS_ENDPOINT = "{scheme}://{bucket_name}.storage.googleapis.com".format(
+                scheme=test_data["scheme"], bucket_name=test_data["bucket"]
+            )
+        resource = "/{}".format(test_data["object"])
+        _run_conformance_test(resource, test_data, _API_ACCESS_ENDPOINT)
+    else:
+        resource = "/{}/{}".format(test_data["bucket"], test_data["object"])
+        _run_conformance_test(resource, test_data)
 
 
 def _make_credentials(signer_email=None):
